@@ -7,21 +7,23 @@
 //!
 //! The buffer reader has also not implemented, but the chunks can be read by sub-parsers.
 //!
-//! Function [`parse`] is used to parse YAML string into [`Node`] data structure.
+//! Function [`parse`] is used to parse YAML string into [`Node`] data structure,
+//! which has a data holder [`Yaml`].
+//! There also has multiple thread version corresponding to [`parse_arc`], [`ArcNode`], [`ArcYaml`].
 //! To get back as string, please use [`dump`] function.
 //!
-//! There are also have some macros for building [`Node`] structure from Rust data.
-//! Especially [`node!`] macro, almost data can be built by the macro literally.
+//! There are also have some macros for building [`NodeBase`] structure from Rust data.
+//! Especially [`node!`] / [`node_arc!`] macro, almost data can be built by the macro literally.
 //!
 //! If you went to rise your own error message, [`indicated_msg`] might be a good choice.
 //!
-//! Please be aware that the anchor system must be done by your self to prevent recursive problem.
-//! This crate is only store the anchor information in [`Yaml::Anchor`] and [`Node::anchor`].
-//! A reference counter system maybe the best choice.
+//! The anchor system is implemented by using [`std::rc::Rc`] and [`std::sync::Arc`] as inner handler,
+//! please see [`AnchorVisitor`].
 pub use crate::dumper::dump;
 pub use crate::indicator::*;
 pub use crate::node::*;
-pub use crate::parser::parse;
+pub use crate::parser::{parse, parse_arc};
+pub use crate::visitor::*;
 pub use crate::yaml::*;
 
 /// Create [`Node`] items literally.
@@ -34,8 +36,6 @@ pub use crate::yaml::*;
 /// assert_eq!(node!(k), node!("a"));
 /// ```
 ///
-/// The members are ordered as `node!(yaml, pos, anchor, ty)`.
-///
 /// Arrays and maps can be created from this macro directly through brackets (`[]`, `{}`).
 ///
 /// ```
@@ -44,33 +44,53 @@ pub use crate::yaml::*;
 /// assert_eq!(node!({node!(1) => node!(2)}), node!(yaml_map![node!(1) => node!(2)]));
 /// ```
 ///
-/// The [`Yaml::Null`] and the [`Yaml::Null`] are also supported by the syntax:
+/// The [`YamlBase::Null`] and the [`YamlBase::Null`] are also supported by the syntax:
 ///
 /// ```
-/// use yaml_peg::{node, Yaml};
-/// assert_eq!(node!(Yaml::Null), node!(null));
-/// assert_eq!(node!(Yaml::Anchor("x".into())), node!(*("x")));
+/// use yaml_peg::{node, YamlBase};
+/// assert_eq!(node!(YamlBase::Null), node!(null));
+/// assert_eq!(node!(YamlBase::Anchor("x".into())), node!(*"x"));
 /// ```
 #[macro_export]
 macro_rules! node {
-    ([$($token:tt)*] $($opt:tt)*) => {
-        $crate::node!($crate::yaml_array![$($token)*] $($opt)*)
+    ([$($token:tt)*]) => {
+        $crate::node!($crate::yaml_array![$($token)*])
     };
-    ({$($token:tt)*} $($opt:tt)*) => {
-        $crate::node!($crate::yaml_map![$($token)*] $($opt)*)
+    ({$($token:tt)*}) => {
+        $crate::node!($crate::yaml_map![$($token)*])
     };
-    (null $($opt:tt)*) => {
-        $crate::node!($crate::Yaml::Null $($opt)*)
+    (null) => {
+        $crate::node!($crate::Yaml::Null)
     };
-    (*($anchor:expr) $($opt:tt)*) => {
-        $crate::node!($crate::Yaml::Anchor($anchor.into()) $($opt)*)
+    (*$anchor:expr) => {
+        $crate::node!($crate::Yaml::Anchor($anchor.into()))
     };
-    ($yaml:expr $(, $pos:expr $(, $anchor:expr $(, $ty:expr)?)?)?) => {
-        $crate::Node::new($yaml.into())$(.pos($pos.into())$(.anchor($anchor.into())$(.ty($ty.into()))?)?)?
+    ($yaml:expr) => {
+        $crate::Node::new($yaml.into(), 0, "", "")
     };
 }
 
-/// Create [`Yaml::Array`] items literally.
+/// [`node!`] macro for [`ArcNode`].
+#[macro_export]
+macro_rules! node_arc {
+    ([$($token:tt)*]) => {
+        $crate::node_arc!($crate::yaml_array![$($token)*])
+    };
+    ({$($token:tt)*}) => {
+        $crate::node_arc!($crate::yaml_map![$($token)*])
+    };
+    (null) => {
+        $crate::node_arc!($crate::Yaml::Null)
+    };
+    (*$anchor:expr) => {
+        $crate::node_arc!($crate::Yaml::Anchor($anchor.into()))
+    };
+    ($yaml:expr) => {
+        $crate::ArcNode::new($yaml.into(), 0, "", "")
+    };
+}
+
+/// Create [`YamlBase::Array`] items literally.
 ///
 /// ```
 /// use yaml_peg::{node, yaml_array};
@@ -83,7 +103,7 @@ macro_rules! yaml_array {
     };
 }
 
-/// Create [`Yaml::Map`] items literally.
+/// Create [`YamlBase::Map`] items literally.
 ///
 /// ```
 /// use yaml_peg::{node, yaml_map};
@@ -107,6 +127,8 @@ pub mod dumper;
 mod indicator;
 mod node;
 pub mod parser;
+pub mod repr;
 #[cfg(test)]
 mod tests;
+mod visitor;
 mod yaml;
