@@ -1,7 +1,7 @@
 use crate::*;
 use std::{
     fmt::{Debug, Formatter, Result as FmtResult},
-    hash::{Hash, Hasher},
+    hash::Hash,
     ops::Index,
     str::FromStr,
 };
@@ -12,10 +12,10 @@ macro_rules! as_method {
         $(| $ty2:ident)* -> $r:ty} => {
         $(#[$meta])*
         pub fn $id(&self) -> Result<$r, u64> {
-            match &self.yaml {
-                Yaml::$ty(v) $(| Yaml::$ty2(v))* => Ok($($op)?v),
-                $(Yaml::Null => Ok($default),)?
-                _ => Err(self.pos),
+            match self.yaml() {
+                YamlBase::$ty(v) $(| YamlBase::$ty2(v))* => Ok($($op)?v),
+                $(YamlBase::Null => Ok($default),)?
+                _ => Err(self.pos()),
             }
         }
     };
@@ -28,28 +28,34 @@ macro_rules! as_num_method {
         where
             N: FromStr,
         {
-            match &self.yaml {
-                Yaml::$ty1(n) $(| Yaml::$ty2(n))* => match n.parse() {
+            match self.yaml() {
+                YamlBase::$ty1(n) $(| YamlBase::$ty2(n))* => match n.parse() {
                     Ok(v) => Ok(v),
-                    Err(_) => Err(self.pos),
+                    Err(_) => Err(self.pos()),
                 },
-                _ => Err(self.pos),
+                _ => Err(self.pos()),
             }
         }
     };
 }
 
-/// Parser node, includes line number, column number, type assertion and anchor.
+/// A node with [`std::rc::Rc`] holder.
+pub type Node = NodeBase<repr::RcRepr>;
+/// A node with [`std::sync::Arc`] holder.
+pub type ArcNode = NodeBase<repr::ArcRepr>;
+
+/// Readonly node, including line number, column number, type assertion and anchor.
+/// You can access [`YamlBase`] type through [`NodeBase::yaml`] method.
 ///
-/// This type will ignore additional members when comparison and hashing.
+/// This type will ignore additional information when comparison and hashing.
 ///
 /// ```
 /// use std::collections::HashSet;
 /// use yaml_peg::Node;
 /// let mut s = HashSet::new();
-/// s.insert(Node::new("a".into()).pos(0));
-/// s.insert(Node::new("a".into()).pos(1));
-/// s.insert(Node::new("a".into()).pos(2));
+/// s.insert(Node::new("a".into(), 0, "", ""));
+/// s.insert(Node::new("a".into(), 1, "my-type", ""));
+/// s.insert(Node::new("a".into(), 2, "", "my-anchor"));
 /// assert_eq!(s.len(), 1);
 /// ```
 ///
@@ -90,52 +96,57 @@ macro_rules! as_num_method {
 /// }
 /// ```
 ///
-/// For default value on map type, [`Node::get`] method has a shorten method [`Node::get_default`] to combining
+/// For default value on map type, [`NodeBase::get`] method has a shorten method [`NodeBase::get_default`] to combining
 /// transform function and default function as well.
-#[derive(Eq, Clone)]
-pub struct Node {
-    /// Document position
-    pub pos: u64,
-    /// Type assertion
-    pub ty: String,
-    /// Anchor reference
-    pub anchor: String,
-    /// YAML data
-    pub yaml: Yaml,
-}
+///
+/// # Clone
+///
+/// Since the YAML data is wrapped by reference counter [`std::rc::Rc`] and [`std::sync::Arc`],
+/// cloning node just increase the reference counter,
+/// the entire data structure are still shared together.
+///
+/// If you want to copy data, please get the data first.
+#[derive(Hash, Eq, PartialEq, Clone)]
+pub struct NodeBase<R: repr::Repr>(R);
 
-impl Node {
+impl<Repr: repr::Repr> NodeBase<Repr> {
     /// Create node from YAML data.
-    pub fn new(yaml: Yaml) -> Self {
-        Self {
-            pos: 0,
-            ty: "".into(),
-            anchor: "".into(),
-            yaml,
-        }
+    pub fn new(yaml: YamlBase<Repr>, pos: u64, ty: &str, anchor: &str) -> Self {
+        Self(Repr::repr(yaml, pos, ty.to_owned(), anchor.to_owned()))
     }
 
-    /// Builder function for position.
-    pub fn pos(mut self, pos: u64) -> Self {
-        self.pos = pos;
-        self
+    /// Document position.
+    #[inline(always)]
+    pub fn pos(&self) -> u64 {
+        self.0.pos()
     }
 
-    /// Builder function for type assertion.
-    pub fn ty(mut self, ty: String) -> Self {
-        self.ty = ty;
-        self
+    /// Type assertion.
+    #[inline(always)]
+    pub fn ty(&self) -> &str {
+        self.0.ty()
     }
 
-    /// Builder function for anchor annotation.
-    pub fn anchor(mut self, anchor: String) -> Self {
-        self.anchor = anchor;
-        self
+    /// Anchor reference.
+    #[inline(always)]
+    pub fn anchor(&self) -> &str {
+        self.0.anchor()
+    }
+
+    /// YAML data.
+    #[inline(always)]
+    pub fn yaml(&self) -> &YamlBase<Repr> {
+        self.0.yaml()
+    }
+
+    /// Drop the node and get the YAML data.
+    pub fn into_yaml(self) -> YamlBase<Repr> {
+        self.0.into_yaml()
     }
 
     /// Check the value is null.
     pub fn is_null(&self) -> bool {
-        self.yaml == Yaml::Null
+        *self.yaml() == YamlBase::Null
     }
 
     as_method! {
@@ -207,12 +218,12 @@ impl Node {
     /// assert!(node!(null).as_value().unwrap().is_empty());
     /// ```
     pub fn as_value(&self) -> Result<&str, u64> {
-        match &self.yaml {
-            Yaml::Str(s) | Yaml::Int(s) | Yaml::Float(s) => Ok(s),
-            Yaml::Bool(true) => Ok("true"),
-            Yaml::Bool(false) => Ok("false"),
-            Yaml::Null => Ok(""),
-            _ => Err(self.pos),
+        match self.yaml() {
+            YamlBase::Str(s) | YamlBase::Int(s) | YamlBase::Float(s) => Ok(s),
+            YamlBase::Bool(true) => Ok("true"),
+            YamlBase::Bool(false) => Ok("false"),
+            YamlBase::Null => Ok(""),
+            _ => Err(self.pos()),
         }
     }
 
@@ -238,7 +249,7 @@ impl Node {
         ///     node!([node!(1), node!(2)]).as_array().unwrap()
         /// );
         /// ```
-        fn as_array = Array -> &Array
+        fn as_array = Array -> &Array<Repr>
     }
 
     as_method! {
@@ -253,7 +264,7 @@ impl Node {
         ///     node!({node!(1) => node!(2)}).as_map().unwrap().get(&node!(1)).unwrap()
         /// );
         /// ```
-        fn as_map = Map -> &Map
+        fn as_map = Map -> &Map<Repr>
     }
 
     /// Convert to map and try to get the value by keys recursivly.
@@ -269,28 +280,28 @@ impl Node {
     /// ```
     pub fn get<Y>(&self, keys: &[Y]) -> Result<&Self, u64>
     where
-        Y: Into<Yaml> + Copy,
+        Y: Into<YamlBase<Repr>> + Copy,
     {
         if keys.is_empty() {
             panic!("invalid search!");
         }
-        match &self.yaml {
-            Yaml::Map(m) => {
-                if let Some(n) = m.get(&node!(keys[0])) {
+        match self.yaml() {
+            YamlBase::Map(m) => {
+                if let Some(n) = m.get(&Self::from(keys[0].into())) {
                     if keys[1..].is_empty() {
                         Ok(n)
                     } else {
                         n.get(&keys[1..])
                     }
                 } else {
-                    Err(self.pos)
+                    Err(self.pos())
                 }
             }
-            _ => Err(self.pos),
+            _ => Err(self.pos()),
         }
     }
 
-    /// Same as [`Node::get`] but provide default value if the key is missing.
+    /// Same as [`NodeBase::get`] but provide default value if the key is missing.
     /// For this method, a transform method `as_*` is required.
     ///
     /// + If the value exist, return the value.
@@ -298,34 +309,34 @@ impl Node {
     /// + If the value is not exist, return the default value.
     ///
     /// ```
-    /// use yaml_peg::{node, Node};
+    /// use yaml_peg::{node, NodeBase};
     /// let a = node!({node!("a") => node!({node!("b") => node!("c")})});
     /// assert_eq!(
     ///     "c",
-    ///     a.get_default(&["a", "b"], "d", Node::as_str).unwrap()
+    ///     a.get_default(&["a", "b"], "d", NodeBase::as_str).unwrap()
     /// );
     /// let b = node!({node!("a") => node!({})});
     /// assert_eq!(
     ///     "d",
-    ///     b.get_default(&["a", "b"], "d", Node::as_str).unwrap()
+    ///     b.get_default(&["a", "b"], "d", NodeBase::as_str).unwrap()
     /// );
     /// let c = node!({node!("a") => node!({node!("b") => node!(20.)})});
     /// assert_eq!(
     ///     Err(0),
-    ///     c.get_default(&["a", "b"], "d", Node::as_str)
+    ///     c.get_default(&["a", "b"], "d", NodeBase::as_str)
     /// );
     /// ```
     pub fn get_default<'a, Y, R, F>(&'a self, keys: &[Y], default: R, factory: F) -> Result<R, u64>
     where
-        Y: Into<Yaml> + Copy,
+        Y: Into<YamlBase<Repr>> + Copy,
         F: Fn(&'a Self) -> Result<R, u64>,
     {
         if keys.is_empty() {
             panic!("invalid search!");
         }
-        match &self.yaml {
-            Yaml::Map(m) => {
-                if let Some(n) = m.get(&node!(keys[0])) {
+        match self.yaml() {
+            YamlBase::Map(m) => {
+                if let Some(n) = m.get(&Self::from(keys[0].into())) {
                     if keys[1..].is_empty() {
                         factory(n)
                     } else {
@@ -335,51 +346,45 @@ impl Node {
                     Ok(default)
                 }
             }
-            _ => Err(self.pos),
+            _ => Err(self.pos()),
         }
     }
 }
 
-impl Debug for Node {
+impl<R: repr::Repr> Debug for NodeBase<R> {
     fn fmt(&self, f: &mut Formatter<'_>) -> FmtResult {
-        f.write_fmt(format_args!("Node{:?}", &self.yaml))
+        f.write_fmt(format_args!("Node{:?}", &self.0))
     }
 }
 
-impl Hash for Node {
-    fn hash<H: Hasher>(&self, state: &mut H) {
-        self.yaml.hash(state)
-    }
-}
-
-impl PartialEq for Node {
-    fn eq(&self, rhs: &Self) -> bool {
-        self.yaml == rhs.yaml
-    }
-}
-
-impl Index<usize> for Node {
+impl<R: repr::Repr> Index<usize> for NodeBase<R> {
     type Output = Self;
 
     fn index(&self, index: usize) -> &Self::Output {
-        match &self.yaml {
-            Yaml::Array(a) => a.get(index).unwrap_or(self),
-            Yaml::Map(m) => m
-                .get(&Node::new(Yaml::Int(index.to_string())))
+        match self.yaml() {
+            YamlBase::Array(a) => a.get(index).unwrap_or(self),
+            YamlBase::Map(m) => m
+                .get(&Self::from(YamlBase::Int(index.to_string())))
                 .unwrap_or(self),
             _ => self,
         }
     }
 }
 
-impl Index<&str> for Node {
+impl<R: repr::Repr> Index<&str> for NodeBase<R> {
     type Output = Self;
 
     fn index(&self, index: &str) -> &Self::Output {
-        if let Yaml::Map(m) = &self.yaml {
-            m.get(&node!(index)).unwrap_or(self)
+        if let YamlBase::Map(m) = self.yaml() {
+            m.get(&Self::from(YamlBase::from(index))).unwrap_or(self)
         } else {
             self
         }
     }
 }
+
+impl<R: repr::Repr> From<YamlBase<R>> for NodeBase<R> {
+    fn from(yaml: YamlBase<R>) -> Self {
+        Self::new(yaml, 0, "", "")
+    }
+}
