@@ -4,25 +4,34 @@ use std::{fmt::Display, iter::FromIterator};
 
 macro_rules! yaml_from_method {
     ($from_ty1:ty $(| $from_ty2:ty)* as $ty:ident) => {
-        impl From<$from_ty1> for Yaml {
+        impl<R: repr::Repr> From<$from_ty1> for YamlBase<R> {
             fn from(s: $from_ty1) -> Self {
-                Yaml::$ty(format!("{}", s))
+                Self::$ty(format!("{}", s))
             }
         }
         $(
-        impl From<$from_ty2> for Yaml {
+        impl<R: repr::Repr> From<$from_ty2> for YamlBase<R> {
             fn from(s: $from_ty2) -> Self {
-                Yaml::$ty(format!("{}", s))
+                Self::$ty(format!("{}", s))
             }
         }
         )*
     };
 }
 
+/// A YAML data with [`std::rc::Rc`] holder.
+pub type Yaml = YamlBase<repr::RcRepr>;
+/// A YAML data with [`std::sync::Arc`] holder.
+pub type ArcYaml = YamlBase<repr::ArcRepr>;
 /// The array data structure of YAML.
-pub type Array = Vec<Node>;
+pub type Array<R> = Vec<NodeBase<R>>;
 /// The map data structure of YAML.
-pub type Map = LinkedHashMap<Node, Node>;
+pub type Map<R> = LinkedHashMap<NodeBase<R>, NodeBase<R>>;
+/// Anchor visitor is made by a hash map that you can get the node reference inside.
+///
+/// Since [`Node`] type is holding a reference counter,
+/// the data are just a viewer to the original memory.
+pub type AnchorVisitor<R> = LinkedHashMap<String, NodeBase<R>>;
 
 /// YAML data types, but it is recommended to use [`Node`] for shorten code.
 ///
@@ -37,7 +46,8 @@ pub type Map = LinkedHashMap<Node, Node>;
 /// Also, the iterators can turn into arrays and maps.
 ///
 /// ```
-/// use yaml_peg::{Yaml, yaml_array, yaml_map, node};
+/// use yaml_peg::{Yaml, node};
+/// use yaml_peg::{yaml_array, yaml_map};
 /// use std::iter::FromIterator;
 /// let v = vec![node!(1), node!(2), node!(3)];
 /// assert_eq!(Yaml::from_iter(v), yaml_array![node!(1), node!(2), node!(3)]);
@@ -45,7 +55,7 @@ pub type Map = LinkedHashMap<Node, Node>;
 /// assert_eq!(Yaml::from_iter(m), yaml_map!{node!(1) => node!(2), node!(3) => node!(4)});
 /// ```
 #[derive(Hash, Eq, PartialEq, Debug, Clone)]
-pub enum Yaml {
+pub enum YamlBase<R: repr::Repr> {
     /// Null
     Null,
     /// Boolean
@@ -57,26 +67,26 @@ pub enum Yaml {
     /// String
     Str(String),
     /// Array
-    Array(Array),
+    Array(Array<R>),
     /// Map
-    Map(Map),
+    Map(Map<R>),
     /// Anchor insertion
     Anchor(String),
 }
 
-impl Yaml {
+impl<R: repr::Repr> YamlBase<R> {
     /// Check the anchor is valid.
     pub fn is_valid_anchor<T>(s: T) -> bool
     where
         T: Display,
     {
-        parser::Parser::new(format!("{}", s).as_bytes())
+        parser::Parser::<R>::new(format!("{}", s).as_bytes())
             .identifier()
             .is_ok()
     }
 }
 
-impl From<bool> for Yaml {
+impl<R: repr::Repr> From<bool> for YamlBase<R> {
     fn from(b: bool) -> Self {
         Self::Bool(b)
     }
@@ -86,19 +96,19 @@ yaml_from_method! { &str | String | &String as Str }
 yaml_from_method! { u8 | u16 | u32 | u64 | u128 | i8 | i16 | i32 | i64 | i128 as Int }
 yaml_from_method! { f32 | f64 as Float }
 
-impl FromIterator<Node> for Yaml {
+impl<R: repr::Repr> FromIterator<NodeBase<R>> for YamlBase<R> {
     fn from_iter<T>(iter: T) -> Self
     where
-        T: IntoIterator<Item = Node>,
+        T: IntoIterator<Item = NodeBase<R>>,
     {
         Self::Array(iter.into_iter().collect())
     }
 }
 
-impl FromIterator<(Node, Node)> for Yaml {
+impl<R: repr::Repr> FromIterator<(NodeBase<R>, NodeBase<R>)> for YamlBase<R> {
     fn from_iter<T>(iter: T) -> Self
     where
-        T: IntoIterator<Item = (Node, Node)>,
+        T: IntoIterator<Item = (NodeBase<R>, NodeBase<R>)>,
     {
         Self::Map(iter.into_iter().collect())
     }
